#include "gdt.h"
#include "util/util.h"

static gdt_entry_t	gdt_entries[5];
static gdt_ptr_t	gdt_ptr;

void gdt_init(void) {
	gdt_ptr.size = (sizeof(gdt_entry_t) * 5) - 1;
	gdt_ptr.base = (uint32_t)&gdt_entries[0];
	
	// Initialize GDT Segments Here
	gdt_set_gate(0, 0, 0, 0, 0);
	gdt_set_gate(1, 0, 0xFFFFFFFF, 0x9A, 0xCF);
	gdt_set_gate(2, 0, 0xFFFFFFFF, 0x92, 0xCF);
	gdt_set_gate(3, 0, 0xFFFFFFFF, 0xFA, 0xCF);
	gdt_set_gate(4, 0, 0xFFFFFFFF, 0xF2, 0xCF);
	
	gdt_flush((uint32_t)&gdt_ptr);
}

void gdt_set_gate(uint32_t gate, uint32_t base, uint32_t limit, uint8_t access, uint8_t granularity) {
	if (gate >= 5) return;
	
	memset((void *)&gdt_entries[gate], 0, sizeof(gdt_entry_t));
	
	gdt_entries[gate].base_low		= (uint16_t)(base & 0xFFFF);
	gdt_entries[gate].base_middle	= (uint8_t)((base >> 16) & 0xFF);
	gdt_entries[gate].base_high		= (uint8_t)((base >> 24) & 0xFF);
	gdt_entries[gate].limit_low		= (uint16_t)(limit & 0xFFFF);
	
	gdt_entries[gate].granularity	= (uint8_t)(limit >> 16) & 0x0F;
	gdt_entries[gate].granularity	|= granularity & 0xF0;
	
	gdt_entries[gate].access 		= access;
}