#include "idt.h"
#include "interrupts.h"

#include "util/util.h"
#include "video/video.h"

static idt_entry_t		idt_entries[256];
static idt_ptr_t		idt_ptr;

static void default_ir_handler(void) {
	printf("Test\n");
	
	__asm__("cli");
	__asm__("hlt");
}

void idt_init(void) {
	idt_ptr.size	= sizeof(idt_entry_t) * 256 - 1;
	idt_ptr.base	= (uint32_t)&idt_entries[0];
	
	// Clear all interrupts for now
	memset((void *)&idt_entries[0], 0, sizeof(idt_entry_t) * 256 - 1);
	
	for (int i = 0; i < 256; i++) {
		idt_set_gate(i, (uint32_t)&(*default_ir_handler), 0x08, 0x8E);
	}
	
	// Install the IDT
	idt_flush((uint32_t)&idt_ptr);
}

void idt_set_gate(uint32_t gate, uint32_t base, uint16_t selector, uint8_t attribute) {
	if (gate >= 256) return;
	
	idt_entries[gate].base_low 	= (uint16_t)(base & 0xFFFF);
	idt_entries[gate].base_high = (uint16_t)((base >> 16) & 0xFFFF);
	
	idt_entries[gate].selector	= (uint16_t)selector;
	idt_entries[gate].reserved	= 0;
	idt_entries[gate].attribute = (uint8_t)attribute;
}