#include "interrupts.h"

#include "idt.h"
#include "pic.h"
#include "pit.h"
#include "video/video.h"

// The custom interrupt functions
static isr_handler_t	isr_handlers[256];

void interrupts_init(void) {
	// Initialize the interrupt entry points
	idt_set_gate(0, (uint32_t)&isr0, 0x08, 0x8E);
	idt_set_gate(1, (uint32_t)&isr1, 0x08, 0x8E);
	idt_set_gate(2, (uint32_t)&isr2, 0x08, 0x8E);
	idt_set_gate(3, (uint32_t)&isr3, 0x08, 0x8E);
	idt_set_gate(4, (uint32_t)&isr4, 0x08, 0x8E);
	idt_set_gate(5, (uint32_t)&isr5, 0x08, 0x8E);
	idt_set_gate(6, (uint32_t)&isr6, 0x08, 0x8E);
	idt_set_gate(7, (uint32_t)&isr7, 0x08, 0x8E);
	idt_set_gate(8, (uint32_t)&isr8, 0x08, 0x8E);
	idt_set_gate(9, (uint32_t)&isr9, 0x08, 0x8E);
	idt_set_gate(10, (uint32_t)&isr10, 0x08, 0x8E);
	idt_set_gate(11, (uint32_t)&isr11, 0x08, 0x8E);
	idt_set_gate(12, (uint32_t)&isr12, 0x08, 0x8E);
	idt_set_gate(13, (uint32_t)&isr13, 0x08, 0x8E);
	idt_set_gate(14, (uint32_t)&isr14, 0x08, 0x8E);
	idt_set_gate(15, (uint32_t)&isr15, 0x08, 0x8E);
	idt_set_gate(16, (uint32_t)&isr16, 0x08, 0x8E);
	idt_set_gate(17, (uint32_t)&isr17, 0x08, 0x8E);
	idt_set_gate(18, (uint32_t)&isr18, 0x08, 0x8E);
	idt_set_gate(19, (uint32_t)&isr19, 0x08, 0x8E);
	idt_set_gate(20, (uint32_t)&isr20, 0x08, 0x8E);
	idt_set_gate(21, (uint32_t)&isr21, 0x08, 0x8E);
	idt_set_gate(22, (uint32_t)&isr22, 0x08, 0x8E);
	idt_set_gate(23, (uint32_t)&isr23, 0x08, 0x8E);
	idt_set_gate(24, (uint32_t)&isr24, 0x08, 0x8E);
	idt_set_gate(25, (uint32_t)&isr25, 0x08, 0x8E);
	idt_set_gate(26, (uint32_t)&isr26, 0x08, 0x8E);
	idt_set_gate(27, (uint32_t)&isr27, 0x08, 0x8E);
	idt_set_gate(28, (uint32_t)&isr28, 0x08, 0x8E);
	idt_set_gate(29, (uint32_t)&isr29, 0x08, 0x8E);
	idt_set_gate(30, (uint32_t)&isr30, 0x08, 0x8E);
	idt_set_gate(31, (uint32_t)&isr31, 0x08, 0x8E);
	
	idt_set_gate(32, (uint32_t)&irq0, 0x08, 0x8E);
	idt_set_gate(33, (uint32_t)&irq1, 0x08, 0x8E);
	idt_set_gate(34, (uint32_t)&irq2, 0x08, 0x8E);
	idt_set_gate(35, (uint32_t)&irq3, 0x08, 0x8E);
	idt_set_gate(36, (uint32_t)&irq4, 0x08, 0x8E);
	idt_set_gate(37, (uint32_t)&irq5, 0x08, 0x8E);
	idt_set_gate(38, (uint32_t)&irq6, 0x08, 0x8E);
	idt_set_gate(39, (uint32_t)&irq7, 0x08, 0x8E);
	idt_set_gate(40, (uint32_t)&irq8, 0x08, 0x8E);
	idt_set_gate(41, (uint32_t)&irq9, 0x08, 0x8E);
	idt_set_gate(42, (uint32_t)&irq10, 0x08, 0x8E);
	idt_set_gate(43, (uint32_t)&irq11, 0x08, 0x8E);
	idt_set_gate(44, (uint32_t)&irq12, 0x08, 0x8E);
	idt_set_gate(45, (uint32_t)&irq13, 0x08, 0x8E);
	idt_set_gate(46, (uint32_t)&irq14, 0x08, 0x8E);
	idt_set_gate(47, (uint32_t)&irq15, 0x08, 0x8E);
	
	// TODO: Initialize each interrupt's custom function
	// NOTE: If the custom function is not registered, the OS will crash
	register_interrupt_handler(0, divide_by_zero_fault);
	register_interrupt_handler(1, general_protection_fault);
	register_interrupt_handler(2, general_protection_fault);
	register_interrupt_handler(3, general_protection_fault);
	register_interrupt_handler(4, general_protection_fault);
	register_interrupt_handler(5, general_protection_fault);
	register_interrupt_handler(6, general_protection_fault);
	register_interrupt_handler(7, general_protection_fault);
	register_interrupt_handler(8, double_fault_abort);
	register_interrupt_handler(9, general_protection_fault);
	register_interrupt_handler(10, general_protection_fault);
	register_interrupt_handler(11, general_protection_fault);
	register_interrupt_handler(12, general_protection_fault);
	register_interrupt_handler(13, general_protection_fault);
	register_interrupt_handler(14, general_protection_fault);
	register_interrupt_handler(15, general_protection_fault);
	register_interrupt_handler(16, general_protection_fault);
	register_interrupt_handler(17, general_protection_fault);
	register_interrupt_handler(18, general_protection_fault);
	register_interrupt_handler(19, general_protection_fault);
	register_interrupt_handler(20, general_protection_fault);
	register_interrupt_handler(21, general_protection_fault);
	register_interrupt_handler(22, general_protection_fault);
	register_interrupt_handler(23, general_protection_fault);
	register_interrupt_handler(24, general_protection_fault);
	register_interrupt_handler(25, general_protection_fault);
	register_interrupt_handler(26, general_protection_fault);
	register_interrupt_handler(27, general_protection_fault);
	register_interrupt_handler(28, general_protection_fault);
	register_interrupt_handler(29, general_protection_fault);
	register_interrupt_handler(30, general_protection_fault);
	register_interrupt_handler(31, general_protection_fault);
	
	register_interrupt_handler(32, timer_interrupt);
	register_interrupt_handler(33, keyboard_interrupt);
	
	register_interrupt_handler(33, general_protection_fault);
	register_interrupt_handler(34, general_protection_fault);
	register_interrupt_handler(35, general_protection_fault);
	register_interrupt_handler(36, general_protection_fault);
	register_interrupt_handler(37, general_protection_fault);
	register_interrupt_handler(38, general_protection_fault);
	register_interrupt_handler(39, general_protection_fault);
	register_interrupt_handler(40, general_protection_fault);
	register_interrupt_handler(41, general_protection_fault);
	register_interrupt_handler(42, general_protection_fault);
	register_interrupt_handler(43, general_protection_fault);
	register_interrupt_handler(44, general_protection_fault);
	register_interrupt_handler(45, general_protection_fault);
	register_interrupt_handler(46, general_protection_fault);
	register_interrupt_handler(47, general_protection_fault);
}

void register_interrupt_handler(uint32_t interrupt, isr_handler_t handler) {
	if (interrupt >= 256) return;
	
	isr_handlers[interrupt] = handler;
}

uint32_t isr_main_entry(uint32_t esp) {
	registers_t * regs = (registers_t *)esp;
		
	// Perform our custom ISR handler
	isr_handler_t handler = isr_handlers[regs->int_no];
	handler(regs);
	
	if (regs->int_no >= 32 && regs->int_no <= 47) {
		// This is an IRQ
		pic_send_eoi(47 - regs->int_no);
	}
	
	return (esp);
}

void print_registers(registers_t * regs) {
	// Print register trace here
	printf("\n");
	printf("\tEAX: %x | EBX: %x | ECX: %x | EDX: %x\n", regs->eax, regs->ebx, regs->ecx, regs->edx);	
	printf("\tEDI: %x | ESI: %x | EBP: %x\n", regs->edi, regs->esi, regs->ebp);
	printf("\tGS:  %x | FS:  %x | ES:  %x | DS:  %x\n", regs->gs, regs->fs, regs->es, regs->ds);
	printf("\tEIP: %x | CS:  %x | ESP: %x | SS:  %x\n", regs->eip, regs->cs, regs->esp, regs->ss);
	printf("\tEFLAGS: %x\n", regs->eflags);
	printf("\n");
}

// MARK: Custom ISR Handlers Here
void divide_by_zero_fault(registers_t * regs) {
	printf("[FAULT]: Program attempted to divide by zero at DIV/IDIV instruction %x\n", regs->eip);
}

void double_fault_abort(registers_t * regs) {
	printf("[ABORT]: DarkOS received a double fault! Aborting...\n");
	
	print_registers(regs);
	
	__asm__ volatile ("cli");
	__asm__ volatile ("hlt");
}

void general_protection_fault(registers_t * regs) {
	printf("[FAULT]: General Protection Fault at address %x (Error: %x)\n", regs->eip, regs->err_code);
	
	print_registers(regs);
	
	__asm__ volatile ("cli");
	__asm__ volatile ("hlt");
}

// MARK: IRQs

void timer_interrupt(registers_t * regs) {
	// THIS WORKS YEEEEEET!!!!!
	pit_update();
}

void keyboard_interrupt(registers_t * regs) {
	printf("Received Keyboard Interrupt\n");
}