#ifndef INTERRUPTS_H
#define INTERRUPTS_H

#include "registers.h"
#include "util/types.h"

// This is a typedef for our interrupt handler function
typedef void (*isr_handler_t)(registers_t *);

// Main functions
void interrupts_init(void);
uint32_t isr_main_entry(uint32_t esp);
void register_interrupt_handler(uint32_t interrupt, isr_handler_t handler);

void print_registers(registers_t * regs);

// Custom Interrupt functions
void divide_by_zero_fault(registers_t * regs);
void double_fault_abort(registers_t * regs);
void general_protection_fault(registers_t * regs);

void timer_interrupt(registers_t * regs);
void keyboard_interrupt(registers_t * regs);

// The entry points for each ISR
extern void isr0(void);
extern void isr1(void);
extern void isr2(void);
extern void isr3(void);
extern void isr4(void);
extern void isr5(void);
extern void isr6(void);
extern void isr7(void);
extern void isr8(void);
extern void isr9(void);
extern void isr10(void);
extern void isr11(void);
extern void isr12(void);
extern void isr13(void);
extern void isr14(void);
extern void isr15(void);
extern void isr16(void);
extern void isr17(void);
extern void isr18(void);
extern void isr19(void);
extern void isr20(void);
extern void isr21(void);
extern void isr22(void);
extern void isr23(void);
extern void isr24(void);
extern void isr25(void);
extern void isr26(void);
extern void isr27(void);
extern void isr28(void);
extern void isr29(void);
extern void isr30(void);
extern void isr31(void);

extern void irq0(void);
extern void irq1(void);
extern void irq2(void);
extern void irq3(void);
extern void irq4(void);
extern void irq5(void);
extern void irq6(void);
extern void irq7(void);
extern void irq8(void);
extern void irq9(void);
extern void irq10(void);
extern void irq11(void);
extern void irq12(void);
extern void irq13(void);
extern void irq14(void);
extern void irq15(void);

#endif