%macro ISR 1
	global	isr%1
	isr%1:
		%if (%1 != 8) && (%1 != 17) && (%1 < 10 || %1 > 14)
		push	dword 0
		%endif
		push	dword %1
		jmp		isr_pre_entry
%endmacro

%macro IRQ 2
	global	irq%1
	irq%1:
		push	dword 0
		push	dword %2
		jmp		isr_pre_entry
%endmacro

; The various ISRs
ISR	0
ISR 1
ISR 2
ISR 3
ISR 4
ISR 5
ISR 6
ISR 7
ISR	8
ISR 9
ISR 10
ISR 11
ISR 12
ISR	13
ISR 14
ISR 15
ISR 16
ISR 17
ISR 18
ISR 19
ISR 20
ISR 21
ISR 22
ISR 23
ISR 24
ISR 25
ISR 26
ISR 27
ISR 28
ISR 29
ISR 30
ISR 31

; The various IRQs
IRQ	0,	32
IRQ 1, 	33
IRQ 2,	34
IRQ	3,	35
IRQ	4,	36
IRQ 5,	37
IRQ	6,	38
IRQ	7,	39
IRQ	8,	40
IRQ	9,	41
IRQ	10,	42
IRQ	11,	43
IRQ	12,	44
IRQ	13,	45
IRQ	14,	46
IRQ	15,	47

extern	isr_main_entry
isr_pre_entry:
	; At this point, the stack has the following pushed (top to bottom)
	; Error Code, Return EIP, Return CS, EFLAGS
	
	; What we want to do now is push the register states onto the stack so that each
	; custom interrupt handler can see the register states when they decide to view
	; what is going on
	
	push	eax
	push	ecx
	push	edx
	push	ebx
	
	push	ebp
	push	esi
	push	edi
	
	push	ds
	push	es
	push	fs
	push	gs
	
	mov		ax, 0x10		; Perform the interrupt in our kernel space
	mov		ds, ax
	mov		es, ax
	mov		fs, ax
	mov		gs, ax
	
	push	esp					
	call	isr_main_entry
	mov		esp, eax
	
	pop		gs
	pop		fs
	pop		es
	pop		ds
	
	pop		edi
	pop		esi
	pop		ebp
	
	pop		ebx
	pop		edx
	pop		ecx
	pop		eax

	add		esp, 8
	iret