#include "pic.h"
#include "util/util.h"

void pic_init(void) {
	pic_remap_irqs();
}

void pic_remap_irqs(void) {
	// Save masks
	uint8_t a1 = inportb(PIC1_DATA);
	uint8_t a2 = inportb(PIC2_DATA);
	
	outportb(PIC1_COMMAND, ICW1_INIT | ICW1_ICW4);
	outportb(PIC2_COMMAND, ICW1_INIT | ICW1_ICW4);
	
	// Set the IRQs to ISR 32 - 47
	outportb(PIC1_DATA, 0x20);
	outportb(PIC2_DATA, 0x28);
	
	outportb(PIC1_DATA, 4);
	outportb(PIC2_DATA, 2);
	
	outportb(PIC1_DATA, ICW4_8086);
	outportb(PIC2_DATA, ICW4_8086);
	
	// Restore saved IRQ masks
	outportb(PIC1_DATA, a1);
	outportb(PIC2_DATA, a2);
}

void pic_send_eoi(uint8_t irq) {
	if (irq >= 8) {
		outportb(PIC2_COMMAND, 0x20);
	}
	
	outportb(PIC1_COMMAND, 0x20);
}

// Masks
void irq_set_mask(uint8_t irq) {
	uint16_t port = (irq < 8) ? PIC1_DATA : PIC2_DATA;
	if (irq >= 8) irq -= 8;
	
	uint8_t value = inportb(port) | (1 << irq);
	outportb(port, value);
}

void irq_clear_mask(uint8_t irq) {
	uint16_t port = (irq < 8) ? PIC1_DATA : PIC2_DATA;
	if (irq >= 8) irq -= 8;
	
	uint8_t value = inportb(port) & ~(1 << irq);
	outportb(port, value);
}