#include "video.h"
#include "util/args.h"
#include "util/util.h"

// Static variables
static uint8_t cursor_x;		// The X Position of the cursor
static uint8_t cursor_y;		// The Y Position of the cursor

// MARK: Text Functions

void putch(char c) {
	if (c == 0x00) {
		return;
	}
	
	// First, check what the text is
	if (c == '\n') {
		cursor_x = 0;
		cursor_y++;
		
		scroll_screen();
		
		return;
	}
	
	if (c == '\r') {
		cursor_x = 0;
		
		return;
	}
	
	if (c == '\t') {
		if (cursor_x + 4 >= COLUMNS) {
			cursor_x = 0 + COLUMNS - cursor_x;
			cursor_y++;
		} else {
			cursor_x += 4;
		}
		
		scroll_screen();
		return;
	}
	
	uint8_t * vidmem = (uint8_t *)VIDEO_MEMORY + (2 * cursor_x) + (2 * cursor_y * COLUMNS);
	
	*vidmem++ = c;
	*vidmem = WHITE_TEXT;
	
	cursor_x++;
	
	if (cursor_x >= COLUMNS) {
		cursor_x = 0;
		cursor_y++;
	}
	
	scroll_screen();
	
	return;
}

void puts(const char * str, size_t len) {
	if (len < 0) return;
	
	for (; len != 0; len--) {
		putch(*str++);
	}
}

// Prints a formatted strig
void printf(const char * str, ...) {
	if (!str) return;
	
	va_list args;
	va_start(args, str);
	
	size_t i = 0;
	for (i = 0; i < strlen(str); i++) {
		switch (str[i]) {
			case '%': {
				switch (str[i+1]) {
					case 'c': {
						uint8_t c = va_arg(args, uint8_t);
						putch(c);
						i++;
						break;
					}
					
					case 's': {
						char * c = va_arg(args, char *);
						char buf[64];
						strcpy(buf, c);
						puts(buf, strlen(buf));
						i++;
						break;
					}
					
					case 'd':
					case 'i': {
						int c = va_arg(args, int);
						char buf[32] = {0};
						itoa(c, buf, 10);
						puts(buf, strlen(buf));
						i++;
						break;
					}
					
					case 'x':
					case 'X': {
						int c = va_arg(args, int);
						char buf[32] = {0};
						itoa(c, buf, 16);
						
						if (strlen(buf) < 8) {
							for (int j = 0; j < 8 - strlen(buf); j++) {
								putch('0');
							}
							
							puts(buf, strlen(buf));
						} else {
							puts(buf, 8);
						}
						
						i++;
						break;
					}
					
					case '%': {
						putch('%');
						break;
					}
					
					default: {
						return;		// Dunno what to do here :|
					}
				}
				
				break;
			}
			
			default: {
				putch(str[i]);
				break;
			}
		}
	}
	
	va_end(args);
}


// MARK: Screen Functions

void clear_screen(void) {
	uint16_t * vidmem = (uint16_t *)VIDEO_MEMORY;
	
	for (uint32_t i = 0; i < ROWS * COLUMNS; i++) {
		vidmem[i] = 0x0000;
	}
}

void scroll_screen(void) {
	uint16_t * vidmem = (uint16_t *)VIDEO_MEMORY;
	
	// First, check if we need to scroll the screen
	if (cursor_y >= ROWS) {
		uint32_t i = 0;
		
		for (i = 0 * 80; i < 24 * 80; i++) {
			vidmem[i] = vidmem[i + 80];
		}
		
		for (i = 24 * 80; i < 25 * 80; i++) {
			vidmem[i] = 0x0000;
		}
		
		cursor_y = 24;
		cursor_x = 0;
	}
}